@@ -32,6 +32,7 @@ class _RfcItem:
         self.level = level
         self.appendix = appendix
         self.page = 0    # This will be set after buffers are complete!
+        self.used = False
 
 
 class _IrefItem:
@@ -298,6 +299,7 @@ class BaseRfcWriter:
         self.date = date
         self.expire_string = ''
         self.ascii = False
+        self.nbws_cond = u'\u00A0'
 
         # We will refer to the XmlRfc document root as 'r'
         self.xmlrfc = xmlrfc
@@ -338,7 +340,14 @@ class BaseRfcWriter:
         p_counter = str(p_counter)  # This is the paragraph counter
         autoName = 'Section ' + counter + ', Paragraph ' + p_counter
         autoAnchor = 'rfc.section.' + counter + '.p.' + p_counter
-        item = _RfcItem(autoName, autoAnchor, anchor=anchor, toc=toc)
+        item = _RfcItem(autoName, autoAnchor, anchor=anchor, toc=toc, counter=p_counter)
+        self._index.append(item)
+        return item
+
+    def _indexListParagraph(self, p_counter, anchor, toc=False):
+        p_counter = str(p_counter)
+        autoName = 'Paragraph ' + p_counter
+        item = _RfcItem(autoName, '', counter=p_counter, anchor=anchor, toc=toc)
         self._index.append(item)
         return item
 
@@ -346,10 +355,10 @@ class BaseRfcWriter:
                       level=1, appendix=False):
         counter = str(counter)
         if appendix:
-            autoName = 'Appendix ' + counter
+            autoName = 'Appendix' + self.nbws_cond + counter
             autoAnchor = 'rfc.appendix.' + counter
         else:
-            autoName = 'Section ' + counter
+            autoName = 'Section' + self.nbws_cond + counter
             autoAnchor = 'rfc.section.' + counter
         item = _RfcItem(autoName, autoAnchor, counter=counter, title=title, \
                        anchor=anchor, toc=toc, level=level, appendix=appendix)
@@ -363,7 +372,7 @@ class BaseRfcWriter:
             autoAnchor = 'rfc.references'
         else:
             subCounter = str(subCounter)
-            autoName = 'References ' + subCounter
+            autoName = 'References' + self.nbws_cond + subCounter
             autoAnchor = 'rfc.references.' + subCounter
         item = _RfcItem(autoName, autoAnchor, counter=counter, title=title, \
                        anchor=anchor, toc=toc, level=level)
@@ -372,7 +381,7 @@ class BaseRfcWriter:
     
     def _indexFigure(self, counter, title=None, anchor=None, toc=False):
         counter = str(counter)
-        autoName = 'Figure ' + counter
+        autoName = 'Figure' + self.nbws_cond + counter
         autoAnchor = 'rfc.figure.' + counter
         item = _RfcItem(autoName, autoAnchor, title=title, anchor=anchor, \
                        toc=toc)
@@ -381,7 +390,7 @@ class BaseRfcWriter:
         
     def _indexTable(self, counter, title=None, anchor=None, toc=False):
         counter = str(counter)
-        autoName = 'Table ' + counter
+        autoName = 'Table' + self.nbws_cond + counter
         autoAnchor = 'rfc.table.' + counter
         item = _RfcItem(autoName, autoAnchor, title=title, anchor=anchor, \
                        toc=toc)
@@ -657,9 +666,10 @@ class BaseRfcWriter:
 
     def write_figure(self, figure):
         """ Writes <figure> elements """
-        align = figure.attrib.get('align', self.defaults['figure_align'])
+        figure_align = figure.attrib.get('align', self.defaults['figure_align'])
         anchor = figure.attrib.get('anchor')
         title = figure.attrib.get('title', self.defaults['figure_title'])
+        suppress_title = figure.attrib.get('suppress-title', 'false')
 
         # Keep track of count if there is an anchor, or PI was enabled
         if anchor or self.pis['figurecount'] == 'yes':
@@ -676,40 +686,39 @@ class BaseRfcWriter:
         # Write preamble
         preamble = figure.find('preamble')
         if preamble is not None:
-            self.write_t_rec(preamble, align=align)
+            self.write_t_rec(preamble, align=figure_align)
 
         # Write figure with optional delimiter
         delimiter = self.pis['artworkdelimiter']
-        artwork = figure.find('artwork').text
-        # artwork_align = artwork.attrib.get('align', align)
-        # Explicitly use figure alignment
-        artwork_align = align
+        artwork = figure.find('artwork')
+        artwork_align = artwork.attrib.get('align', figure_align)
         blanklines = int(self.pis['artworklines'])
-        self.write_raw(artwork, align=artwork_align,
+        self.write_raw(artwork.text, align=artwork_align,
                        blanklines=blanklines, delimiter=delimiter,
                        source_line=figure.sourceline)
 
         # Write postamble
         postamble = figure.find('postamble')
         if postamble is not None:
-            self.write_t_rec(postamble, align=align)
+            self.write_t_rec(postamble, align=figure_align)
 
         # Write label
         title = figure.attrib.get('title', '')
         if anchor or self.pis['figurecount'] == 'yes':
-            if title:
-                title = ': ' + title
-            self.write_label('Figure ' + str(self.figure_count) + title,
-                            type='figure', source_line=figure.sourceline)
-        else:
-            if title:
-                self.write_label(title, type='figure', source_line=figure.sourceline)
+            if suppress_title == 'false':
+                if title:
+                    title = 'Figure ' + str(self.figure_count) + ': ' + title
+                else:
+                    title = 'Figure ' + str(self.figure_count)
+        if title:
+            self.write_label(title, type='figure', source_line=figure.sourceline)
 
     def write_table(self, table):
         """ Writes <texttable> elements """
         align = table.attrib.get('align', self.defaults['table_align'])
         anchor = table.attrib.get('anchor')
         title = table.attrib.get('title', self.defaults['table_title'])
+        suppress_title = table.attrib.get('suppress-title', 'false')
 
         # Keep track of count if there is an anchor, or PI was enabled
         if anchor or self.pis['tablecount'] == 'yes':
@@ -739,13 +748,13 @@ class BaseRfcWriter:
         # Write label if anchor is set or PI figurecount = yes
         if anchor or self.pis['tablecount'] == 'yes':
             title = table.attrib.get('title', '')
-            if title:
-                title = ': ' + title
-            self.write_label('Table ' + str(self.table_count) + title, \
-                             type='table', source_line=table.sourceline)
-        else:
-            if title:
-                self.write_label(title, type='table', source_line=table.sourceline)
+            if suppress_title == 'false':
+                if title:
+                    title = 'Table ' + str(self.table_count) + ': ' + title
+                else:
+                    title = 'Table ' + str(self.table_count)
+        if title:
+            self.write_label(title, type='table', source_line=table.sourceline)
 
     def _index_t_rec(self, element):
         """ Traverse a <t> element only performing indexing operations """
@@ -954,7 +963,8 @@ class BaseRfcWriter:
         if len(references) == 1:
             refs_title = references[0].attrib.get('title', refs_title)
 
-        self._indexReferences(refs_counter, title=refs_title)
+        if len(references) > 0:
+            self._indexReferences(refs_counter, title=refs_title)
 
         if len(references) > 1:
             for i, reference_list in enumerate(references):
@@ -1063,7 +1073,8 @@ class BaseRfcWriter:
         if len(references) == 1:
             refs_title = references[0].attrib.get('title', refs_title)
 
-        self.write_heading(refs_title, bullet=refs_counter + '.', \
+        if len(references) > 0:
+            self.write_heading(refs_title, bullet=refs_counter + '.', \
                                autoAnchor='rfc.references')
         if len(references) > 1:
             for i, reference_list in enumerate(references):
@@ -1096,6 +1107,7 @@ class BaseRfcWriter:
         for author in authors:
             self.write_address_card(author)
 
+        self.check_for_unused_references()
         
     def write(self, filename, tmpfile=None):
         """ Public method to write the RFC document to a file. """
@@ -1135,6 +1147,14 @@ class BaseRfcWriter:
         if not self.quiet and filename:
             xml2rfc.log.write('Created file', filename)
 
+    def check_for_unused_references(self):
+        """ If this is a reference and it is not used - then warn me
+        """
+        if not self.indexmode:
+            for item in self._index:
+                if item.autoAnchor.startswith("rfc.ref.") and not item.used:
+                    xml2rfc.log.warn("no <xref> in <rfc> targets <reference anchor='%s'>" % item.anchor)
+                
     # -----------------------------------------
     # Base writer interface methods to override
     # -----------------------------------------
