@@ -18,8 +18,23 @@ class ExpandedXmlWriter:
         self.quiet = quiet
         self.verbose = verbose
 
+    def post_process_lines(self, lines):
+        output = [ line.replace(u'\u00A0', ' ') for line in lines ]
+        return output
+
     def write(self, filename):
         """ Public method to write the XML document to a file """
+        # clean out entity definitions first
+
+        element = self.root
+        lines = []
+        while element is not None:
+            lines.append(lxml.etree.tostring(element, pretty_print=True))
+            element = element.getprevious()
+        lines.append("<!DOCTYPE rfc SYSTEM \"rfc2629.dtd\">\n")
+        lines.append("<?xml version=\"1.0\" encoding='utf-8'?>\n")
+        lines.reverse()
+       
         # Use lxml's built-in serialization
         file = open(filename, 'w')
         if sys.version > '3':
@@ -28,10 +43,8 @@ class ExpandedXmlWriter:
                                        encoding='utf-8', 
                                        pretty_print=True).decode())
         else:
-            file.write(lxml.etree.tostring(self.root.getroottree(), 
-                                       xml_declaration=True, 
-                                       encoding='utf-8', 
-                                       pretty_print=True))
+            foo = ''
+            file.write(foo.join(lines))
 
         if not self.quiet:
             xml2rfc.log.write('Created file', filename)
