@@ -35,6 +35,7 @@ class HtmlRfcWriter(BaseRfcWriter):
         BaseRfcWriter.__init__(self, xmlrfc, quiet=quiet, date=date, verbose=verbose)
         self.list_counters = {}
         self.iref_index = []
+        self.nbws_cond = ' '    # Use space as they are in an anchor element anyway
         
         # Initialize templates directory
         self.templates_dir = templates_dir or \
@@ -68,6 +69,14 @@ class HtmlRfcWriter(BaseRfcWriter):
         self.iref_table = None
 
     def write_list(self, list, parent, level=0):
+        # grab the anchors that exist
+        if self.indexmode:
+            t_count = 0
+            for t in list.findall('t'):
+                t_count += 1
+                anchor = t.attrib.get('anchor')
+                if anchor:
+                    self._indexListParagraph(t_count, anchor)
         # style comes from the node if one exists
         style = list.attrib.get('style', '')
         if not style:
@@ -178,6 +187,12 @@ class HtmlRfcWriter(BaseRfcWriter):
         """ Return a list of HTML elements that represent the reference """   
         if element.tag == 'xref':
             target = element.attrib.get('target', '')
+            item = self._getItemByAnchor(target)
+            if not self.indexmode:
+                if not item:
+                    xml2rfc.log.warn("Can't resolve xref target %s" % target)
+                else:
+                    item.used = True
             if element.text:
                 cite = E.CITE('[' + target + ']', title='NONE')
                 a = E.A(element.text, href='#' + target)
@@ -190,9 +205,10 @@ class HtmlRfcWriter(BaseRfcWriter):
                 format = element.attrib.get('format', 
                                             self.defaults['xref_format'])
                 a = E.A(href='#' + target)
-                item = self._getItemByAnchor(target)
-                if not item or format == 'none':
+                if not item:
                     text = '[' + target + ']'
+                elif format == 'none':
+                    text = ''
                 elif format == 'counter':
                     text = item.counter
                 elif format == 'title':
@@ -417,8 +433,9 @@ class HtmlRfcWriter(BaseRfcWriter):
                 street_spans = []
                 for street in postal.findall('street'):
                     if street.text:
-                        span = self._serialize(E.SPAN(street.text))
-                        street_spans.append(span)
+                        span = E.SPAN(street.text)
+                        span.attrib['class'] = 'vcardline'
+                        street_spans.append(self._serialize(span))
                 subs['street_spans'] = ''.join(street_spans)
                 city = postal.find('city')
                 if city is not None and city.text:
@@ -477,11 +494,12 @@ class HtmlRfcWriter(BaseRfcWriter):
         for i, reference in enumerate(list.findall('reference')):
             tr = E.TR()
             # Use anchor or num depending on PI
+            anchor = reference.attrib.get('anchor', str(i + self.ref_start))
             if self.pis['symrefs'] == 'yes':
-                bullet = reference.attrib.get('anchor', str(i + self.ref_start))
+                bullet = anchor
             else:
                 bullet = str(i + self.ref_start)
-            bullet_td = E.TD(E.B('[' + bullet + ']', id=bullet))
+            bullet_td = E.TD(E.B('[' + bullet + ']', id=anchor))
             bullet_td.attrib['class'] = 'reference'
             ref_td = E.TD()
             ref_td.attrib['class'] = 'top'
@@ -514,7 +532,8 @@ class HtmlRfcWriter(BaseRfcWriter):
                     ref_td.append(a)
                     last = a
                 a.tail = ', '
-            last.tail = ', "'
+            if len(authors):
+                last.tail = ', "'
             title = reference.find('front/title')
             if title is not None and title.text:
                 title_string = title.text.strip()
@@ -556,7 +575,7 @@ class HtmlRfcWriter(BaseRfcWriter):
             annotation = reference.find('annotation')
             if annotation is not None and annotation.text:
                 ref_td.append(E.P(annotation.text))
-        if self.pis['sortrefs'] == 'yes':
+        if self.pis['sortrefs'] == 'yes' and self.pis['symrefs'] == 'yes':
             refkeys = sorted(refkeys)
         for key in refkeys:
             tbody.append(refdict[key])
@@ -730,8 +749,13 @@ class HtmlRfcWriter(BaseRfcWriter):
                  'toc_head_links':  ''.join(self.buffers['toc_head_links']),
                  'toc_rows':        ''.join(self.buffers['toc_rows'])
                 }
+        for key in subs.keys():
+            subs[key] = subs[key].encode('ascii', 'xmlcharrefreplace')
         self.output = self.templates['base.html'].substitute(subs)
 
+    def post_process_lines(self, lines):
+        return lines
+
     def write_to_file(self, file):
         file.write(self.output)
 
